@@ -24,7 +24,6 @@ import llfuse
 import logging
 import os
 import signal
-import stat
 import sys
 import tempfile
 import thread
@@ -206,6 +205,9 @@ def main(args=None):
     # Do not update .params yet, dump_metadata() may fail if the database is
     # corrupted, in which case we want to force an fsck.
     param['max_inode'] = db.get_val('SELECT MAX(id) FROM inodes')
+    if operations.failsafe:
+        log.warn('File system errors encountered, marking for fsck.')
+        param['needs_fsck'] = True
     with backend_pool() as backend:
         seq_no = get_seq_no(backend)
         if metadata_upload_thread.db_mtime == os.stat(cachepath + '.db').st_mtime:
@@ -246,17 +248,15 @@ def main(args=None):
     db.close()
 
     if options.profile:
-        tmp = tempfile.NamedTemporaryFile()
-        prof.dump_stats(tmp.name)
-        fh = open('s3ql_profile.txt', 'w')
-        p = pstats.Stats(tmp.name, stream=fh)
-        tmp.close()
-        p.strip_dirs()
-        p.sort_stats('cumulative')
-        p.print_stats(50)
-        p.sort_stats('time')
-        p.print_stats(50)
-        fh.close()
+        with tempfile.NamedTemporaryFile() as tmp, \
+            open('s3ql_profile.txt', 'w') as fh:
+            prof.dump_stats(tmp.name)
+            p = pstats.Stats(tmp.name, stream=fh)
+            p.strip_dirs()
+            p.sort_stats('cumulative')
+            p.print_stats(50)
+            p.sort_stats('time')
+            p.print_stats(50)
 
 def determine_threads(options):
     '''Return optimum number of upload threads'''
@@ -341,15 +341,9 @@ def get_metadata(backend, cachepath):
             return tmpfh
         log.info('Downloading and decompressing metadata...')
         tmpfh = backend.perform_read(do_read, "s3ql_metadata")
-        os.close(os.open(cachepath + '.db.tmp', os.O_RDWR | os.O_CREAT | os.O_TRUNC,
-                         stat.S_IRUSR | stat.S_IWUSR))
-        db = Connection(cachepath + '.db.tmp', fast_mode=True)
         log.info("Reading metadata...")
         tmpfh.seek(0)
-        restore_metadata(tmpfh, db)
-        db.close()
-        os.rename(cachepath + '.db.tmp', cachepath + '.db')
-        db = Connection(cachepath + '.db')
+        db = restore_metadata(tmpfh, cachepath + '.db')
 
     # Increase metadata sequence no 
     param['seq_no'] += 1
@@ -663,7 +657,7 @@ def timed_wait(event, timeout, interval=1):
     which causes noticeable CPU consumption. Therefore, for Python 2 we we 
     implement our own poll loop that runs at a rate of our own choosing.
     '''
-    if sys.version_info[0] > 2:
+    if sys.version_info[0] > 2 or timeout is None:
         return event.wait(timeout)
 
     endtime = time.time() + timeout
